      SUBROUTINE LSC(LhPwrMWx,
     ^               nTSCwrix, nTSCreax, nTSCgrax, nLSCcomx, nTSCunux,
     ^               nTSCscrx,
     ^               iRayTrsx, iPlotsix, iErrorx)
c     LSC  main control module              ---------------------------|
c                                                                      |
c                                                                      |
c     LSC  -- Lower hybrid Simulation Code
c     AcDc -- A Current Drive Code
c     Copyright D. W. Ignat, E. J. Valeo,
c               N. J. Fisch, C. F. F. Karney,
c               S. C. Jardin, S. Bernabei, D. P. Enright.
c     Princeton University
c     1992, 1993, 1994, 1995, 1996, 2000
c
c     References:  
c       D. W. Ignat, Phys. Fluids, <24>, 1110 (81);
c       E. J. Valeo and D. C. Eder, J. Comp. Physics, <69>, 341 (87).
c       C. F. F. Karney and N. J. Fisch, Phys. Fluids <29>, 180 (86).
c       D. W. Ignat, E. J. Valeo, and S. C. Jardin,
c       "Dynamic Modeling of Lower Hybrid Current Drive,"
c       Nuclear Fusion <34>, 837-852 (94)
c       D. W. Ignat,  B. P. LeBlanc, C. K. Phillips, 
c       J. R. Wilson, and R. V. Budny, 
c       "Computational Model for Fast Wave Current Drive,
c       Proc. 11th Conf. on Radio Frequency Power in Plasmas, 
c       (Palm Springs, CA, May 1995)
c       D. W. Ignat, R. Kaita, S. C. Jardin, and M. Okabayashi
c       "Spreading of Lower Hybrid Wave Driven Currents in PBX-M,"
c       Nucl. Fusion <36> 1733-1742 (96)
c       Myunghee Ju, Jinyong Kim, KSTAR Team,
c       "A predictive study of non-inductive current driven
c       KSTAR tokamak discharge modes using a new transport/heating
c       simulation package,"
c       Nucl. Fusion <40> 1859-1866 (2000)
c
c            Warning: Local variables are assumed to be SAVEd by the
c            compiler.  While this is a standard implementation, it is
c            not ANSI standard. 
c
c     LSC_19Nov00
c        1. Change the branching around the no-ops in ezsg.F, XrayCam2.F to
c            #ifndef USE_SGLIB
c             ..no-ops
c            #else
c             ..real code
c            #endif
c            #ifndef USE_X_RAY_CAMERA
c             ..no-ops
c            #else
c             ..real code
c            #endif
c           which are more logically consistent
c     LSC_16Oct00
c        1. Make changes found necessary for running with TSC
c           set nLSCcom2 = nTSCscrn
c           recycle ns30c1 to be nTSCgraf, the unit for gnuplot stuff
c           abandon writes of GrfTitle to ns30c1, or whatever
c        2. Use "call date_and_time(zzdate,zztime,zzzone,ivals)" in gnup.F
c           instead of "call date", "call itime" (or "call clock" for Cray)
c           "date_and_time" is a standard intrinsic in Fortran 90.   But,
c           under Fujitsu Linux an -X9 compiler switch is needed.
c           g77 knows about "date_and_time" .
c        3. Remove spurious "fdate" in XrayCam2.F; add "date_and_time".
c           Remove spurious mention of ezlilo, not used any more.
c     LSC_25Sep00
c        1. Remove unused *.inc : Labels, Diags, local1d, plotf .
c        2. Re-arrange inputs and parameters `graph' for more logical order
c        3. Remove disagreement in VecMnMx over real*4 / real*8
c           (Glitch not previously noticed while ridding code of
c            dependence on the -r8 type compiler switch.)
c        4. Search/fix other similar.  
c           Found in ezsg, agraphdi; Raytrace, GetKscat;
c           jrf, SmoothJrf; Raytrace, ran3
c     LSC_03Aug00
c        0. Pull out Minneapolis/Boulder graphs, testSmooth, SmoothInt
c        1. Correct errors in Jray _graphs_ which eliminated negative J's
c        2. Remove Rayout, mktrajb, mktrajc.  These are ray plots in
c           the vpar-psi plane.  (I did not grasp the difference, surely
c           not obvious on looking at the output.)  These did not use EZ
c           calls, so were getting hard to support.
c        3. Adjust placement of n_\parallel, n_\perp vs rt psi; 
c           3d isometric of ray, poloidal cross section of ray; 
c           Adjust placement of n_\parallel enhancement and accessibility
c        4. Rename iFastPiece to be iOrigMaxwl in Giruzzi of pbxio.F
c           to make code more understandable.
c        5. Recast and simplify all Plot Print flags; see PlPr.inc
c        6. Add GNplot for rays, Jrf, Prf.  Revise SG-only wave 
c           scattering plots to go in the middle of ray plots
c        7. Insert three `dummy' couplers USR1 USR2 USR3 and set them
c           equal to TFTR.  The intent is for these to be modified in the 
c           source.
c        8. Fiddle to get gnuplot filename to take date/time from
c           both Fujitsu f90 and G77.
c        9. Many small changes aimed at getting things to work in
c           double precision without "-r8" switches.
c     LSC_27Jun00
c        0. Work on writes to the main log file, improve readability.
c           Small non-important bug in forgetting path length in ray log,
c           but have not found it yet.  (I remember once understanding the issue.)
c     LSC_21Jun00
c        0. Take out unit 6 writes; DoBern, DoComm,  DoEVsm, DoAdFs
c           DoEVsm is still available through iSMO, see below, but recompile.
c        1. Take out iRememFe nRampOld
c     LSC_12Jun00
c        0. Write Pql Pray and integrals for gnuplot
c     LSC_05Jun00
c        0. Remove phony symmetric hi n_parallel spectrum if DoBram = -1
c           (See 30Apr93)
c        1. Write spectrum to data file for gnuplot (SM ?) post processing
c     LSC_19May00 
c        0. Take out DoRFHe, DoRFDa, DoPaus, DoScrn, DoDeBg, DoGrzi, DoHarv
c        1. Take out epol Err Ert Err2 Ert2 (NJ Fisch used these once)
c     LSC_17May00 
c        0. Change setpwrl to do by default linear ramp up assuming about
c           100 ramp steps, with flat at the top, but keep in the geometric
c           ramp code, both original (2 * per) and trial (1.eps * per).
c           Invent parameter for NFLATDEF; use NRAMPDIM for default nrampup
c        1. Take out graph of Er Ntht Nrho Eth Err
c        2. Fix bug that diffused power can be negative where current is
c           negative.  Just take abs(Jrf) in the weighting formula.
c     LSC_15May00  -- simplify input and remove features not, or seldom, used
c        0. Split NAMELIST files in two pieces, 
c           input.lhh containing  `inpvalue'  --- the most  basic  parameters
c           input.xpt containing  `inpexprt'  --- the experts-only parameters
c        1. Take out the stuff put in for Norton L Bretz ( NLB , bret )
c        2. Take out the stuff put in for Nathaniel J Fisch  (Drt DoFish) .
c        3. Take out the stuff put in for Bob Harvey, but save the code
c        4. Take out the stuff put in for Franco Paoletti, but save the code
c        5. Take out non uniform velocity grids 
c     LSC_21Apr00  -- interim fixes, not a stable version 
c        0.  Redefine rFudgDmp to be the fractional power lost in a zone
c            in the ray picture, and use that fraction constructing Dql
c            Abandon iFudgDmp, since rFudgDmp needed for all zones, rays.
c        1.  Greatly increase the allowed number of iterations on f_e to 
c            200, and set up nflat with kludge to select a build up method:
c              nflat = 1,2 ... nramup-1  ==> old method
c              nflat = nrampup           ==> 1.15 increase each cycle
c              nflat > nrampup           ==> 1/nrampup increase each cycle
c        2.  Change the test for stopping the ray calculation so that
c            the calculation runs longer, and therefore can accomodate
c            large amounts of QL burn through.  It used to be that if
c            Maxwellian damping was ``strong'' then the ray stopped, but
c            now we say that Maxwellian damping cannot be more than 10%
c            per region, so it takes hundreds of strong damping zone crossings
c            to absorb the ray.
c     LSC_03Jan00
c        0.  Add '#include "PlPr.inc"' in SUBROUTINE GrafParm to support
c            printing out of "idiag".  See 07Feb96.  g77 found this for me.  
c            Gasp.  Thank gnu.
c     LSC_07Feb96
c        0.  Add idiag to plot of paramters.  Missing all this time!
c     LSC_16Jan96
c        0.  Take out references to dpsi.  It was not actually
c        1.  Install the EhstKarney formula.
c     LSC_20Sep95
c        0.  Allow diffusing the rf power with PrfSpred if current has
c            been diffused with DiffuJrf such that
c            Praytot(psi) = Prf-total n(psi) J-diffused(psi) dV(psi) / 
c                           [ \Sigma  n(psi) J-diffused(psi) dV(psi) ] *
c                                                 PrfSpred     +
c                  Prf-ray-undiffused(psi)* (1. - PrfSpred) 
c        1.  Add phony listing of power to ions in transfer to calling
c            code; this to help with compatibility with TSC/FCD/TRANSP.
c     LSC_14Jul95
c        0.  Warning: Local variables are assumed to be SAVEd by the
c            compiler.  While this is a standard implementation, it is
c            not ANSI standard.  In particular, the NERSC A machine
c            compiler must be forced to do this with an option switch.
c        1.  call EZfini(0,0) consistently; remove 4-transition-point
c            grid stretcher with calls mkgrid eegrd vecadd and vars
c            frv1minus, frv2minus, v1minus, v2minus,
c            frv1plus,  frv2plus,  v1plus,  v2plus,  epsvgr
c
c     LSC_10Feb95
c        0.  Add coupler 'TFTRLHCD' which is 32 0.55cm wgs, 0.15cm septum
c            Move EXCLUDED to 1.2 from 1.5; better for TFTR
c        1.  Repair bug in the error trap on nrays, ntors, npols;
c            add trap of error nparmin>nparmax
c        2.  Replaced "100." with REXPMIN in a couple of  places in FeMaxw; 
c            this avoids compiler problem under osf/1 v3.0, f77 v3.6:
c            evaluation of exp( -100. ) gave an arithmetic error.
c            In the TRANSP environment, however, it should be noted that 
c            exp( -REXPMIN ) evaluates by silent underflow to zero. --DMC
c        3.  Add time of equilibrium to certain graphs; see EquTitle(81:96)
c     LSC_03Dec94
c        0.  Diffuse Jrf accorting to a method like the Fuchs method.
c            See extensive comments under SmoothJrf. 
c            DiffuJrf in the namelist governs this; REPLACES JrfEnhc
c        1.  Add graphs of Jrf by velocity and psi for Bernabei 94 APS meeting
c            see JrfDiagn
c        2.  Move SUBROUTINE LSC to top of file...ahead of comments
c     LSC_30Sep94
c        0.  Insert conditional compilation for TRANSP
c        1.  Split new modules on the /soft/TRANSP-LSC with fsplit
c     LSC_30Aug94
c        0.  Enter option for JET_LHCD coupler
c        1.  Initialize iLastRy in Block Data as Fortran requires
c        2.  Remove infinite loop potential in RayIni....
c        3.  Add nslices to parameter output graph
c        4.  Trap read errors in RdTSC
c        5.  Trap powers(i) = 0.0; reset to 0.001
c     LSC_23May94
c        0.  Install Do0Edc so the Edc in lhcdoua/jardin.d etc can be ignored
c            and treated as zero.  This to investigate possible stability 
c            problems with T/LSC combined on some shots like 313258
c        1.  Add graph of n// enhancement on midplane to pitch(r), 
c            Ne(r), Pray(r) attached to PlFlg(26)
c     LSC_28Mar94
c        0.  Change the Pitch of the field line graph to degrees; range: +/- 6
c        1.  Add plot of Pray(v) for various psi. Requested by Stefano for the
c            Boulder meeting....to show that PBX is relevant to TPX. Flag 19
c        2.  Also, add plot of Pray(iv) summed over all positions and 
c            Plaunch at the edge in v, nparallel, and 1/nparallel(?) space. 
c     LSC_10Dec93
c        0.  Allow the use of phony foils to see the Photon Temperature
c            as determined at lower energy might be higher.  Foil code
c            'PF' sets up only Aluminum with the thickness given
c        1.  Allow the introduction of a fast particle population such that
c            T/Tfast = epsT, nfast/n = epsN,  nfast Tfast /(nT) = epsP
c            using variables Tail?eps where ? = N, T, P
c        2.  Apply trapezoid rule for the integration over 
c            \nuColl / (Dc + Dql) dv uniformly for +- v and starting and
c            subsequent iterations.
c        3.  Allow for operation on non-symmetric equilibria
c     LSC_18Oct93
c        0.  Allow re-calcluations of rays to take place 1 ray at a time
c            with Do1Rpr=1 (Do 1 Ray per retrace call)  See iLastRy, iDoRay
c        1.  Added entry points to help with TSC restarts:
c            ENTRY PutLSCrestart
c            ENTRY GetLSCrestart
c        2.  Add fast electron population with Doflag DoAdFs.
c            The npsi*nv numbers come from another code GGfe.F
c     LSC_03Sep93
c        0.  Give TSC the power and current !!centered!! on the psi values.
c            Due to a previous misunderstanding this had been given on
c            volume shells ending at the psi values passed.
c        1.  Allow TSC to input directly the new loop voltage and call
c            directly for the new current, and dJ/dE
c              SUBROUTINE JdepFromTSC(curtsc,djdets2,npsiTake) in jrf.F
c              ENTRY PutVinLSC(voltPut,npsiPut)                in grapgrd.F
c              note: location#1 is ignored---first data at second location
c              in LSC  input arrays are 1-->NpsiJ;   in TSC 1 -->npsiPut
c              in LSC output arrays are 1-->NpsiJ-1; in TSC 1 -->npsiTake
c     LSC_31Aug93
c        0.  Add EZdscw EZdscr EZopen EZclos to help capture numbers
c            for later manipulation; employ on vertical xray slice
c        1.  Fix error in paramater-list-graph for nrays
c        2.  work xray units: inten in w/cm^2 on camera; w/cm^3 in plasma;
c            including solid angle effect, see "solAng", and absorbers;
c            make frac = ri - i inst of ri - 1 in interpolator; move Cth0 = mu;
c            fix E-indexing; fix multiplication of normalizations; clamp Emax.
c        3.  Add nRunDot, jRunDot == d/dt {n_e_runaway,j_runaway_from_n_dot}
c            = Dql df/dv * {1; q_e v_parallel} | cooperative runaway velocity
c     LSC_19Jun93
c        0.  Add ability to launch rays in the middle, bypass the ql
c            calculation; for TFTR support as discussed with Norton L. Bretz.
c            DoFlag: DoBret; Starting points rstNLB, zstNLB, npaNLB, degNLB
c            ALL THIS UNDER ``0.'' TAKEN OUT MAY 2000; Sorry, Dr Bretz (NLB)
c        1.  Add ability to write out the psi, r, z, n//, n_rho, etc for
c            Paoletti and Bernabei who are exploring with MatLab the 
c            'Volcano Limit' to n//.
c        2.  Also for Paoletti, add r,z,rtpsi,ompe/omce,Bth,Bph
c !      3.  Fixed bugs in Wr2TSC.F and Rayio.F where Npsi was given instead
c            of NpsiJ as size of normalized flux array, for graphs of 
c            approximate damping and dlnJdlnE
c     LSC_24May93
c        0.  Repair work on code that writes data for Bob Harvey.
c            sene was wrong; exde etc in doubt; wdnpar erratic
c        1.  Properly put $ at end of pline with call to DolAtEnd(pline)
c        2.  Call FastFrac every time at request of TRANSP
c        3.  Add plot of n// enhancement factor in AccesiSB; puncture plot of
c            actual enhancement
c     LSC_30Apr93
c        0.  Allow for a phony symmetric hi n_parallel spectrum if DoBram = -1
c            This is for an IBW synergy simulation, but not with IBW disp rel.
c        1.  Short circuit warnings if iRayTrsi=0
c     LSC_20Apr93
c        0.  Density fluctuation scattering at bounces at edge; graphs w/ rays
c        1.  Eliminate NparEnhc in namelist; add scatKdeg;
c            add GetKscat and ran3 subroutines; modify BounShft extensively
c !!!    2.  Split the npar array into ntor(NTORDIM) npol(NPOLDIM)
c            These form components of npar(NRAYDIM); add nrays=ntors*npols;
c            ntor replaces npargrid; add npolmax/min;
c        3.  Stopped a ray with more than 4 zones of heavy damping 
c            according to a Maxwellian calculation: MxdlnPdz = 0.001 
c            This means no penalty for large number of steps.
c        4.  Moved NZONDIM to 2000; 
c            removed restriction that NVELDIM should be greater than 
c            NPSIDIM + NZONDIM by making WKVDIM = NVELDIM+NPSIDIM+NZONDIM;
c            made plot workspace large enough with MxPLTZON=NPLTDIM+NZONDIM & 
c            moving IDM from 200 to NZONDIM.  These requested by Takahashi
c        5.  Changed LSCwarn to be left justified
c     LSC_29Mar93
c        0.  E. Valeo corrected the integral over absorber function, 
c            and made other changes to the x ray camera. Constants still to be
c            made correct.  Forced nMUbins to be odd, but not sure this is
c            necessary.  Took out an extra density in the sigma.
c        1.  Small adjustments to absorber code reflecting refinements from
c            S. vonGoeler.  Dimensioning error in GetXmiss was fixed.
c     LSC_15Mar93
c        0.  Improved, added graphs having to do with Xray signal and emission
c        1.  Set ISIZElcfs = 800 because 100 was causing errors
c        2.  Made aspect ratio for coutours of flux and accessibility ok 
c     LSC_23Feb93
c        0.  Adjusted Rayio.F on tracing flux surface at 0.3*PsiLim
c     LSC_16Feb93
c        0.  Allowed edge density to be reduced with DoBern=1.  See grapgrd.F
c        1.  Made namelist Inpval an include file at request of TRANSP
c     LSC_06Feb93
c        0.  Improved labeling of launched spectrum graph
c        1.  Trapped and reported possible errors on coupler names
c        2.  Added a very slow coupler, called SLOWSLOW with 16 wgs
c            This not accessible from TRANSP namelist which is integer-driven
c !!!    3.  Corrected error that fghz was not being reported to brambJES !
c     LSC_03Feb93
c        0.  Added plot of power in ray vs root psi, with the first 4 passes
c            at no damping followed in the graph
c        1.  Adjusted step in psi surface plotter to handle D-III-D extremes.
c            Also adjusted start search point to be 0.9 of Rmag
c        2.  Took out DoRFDa and DoRFHe as functioning; left in namelist
c            but took out of include file Doflags.inc
c     LSC_28Jan
c        0.  Made SMALL in fe.F 2.5e-05 for TRANSP use.
c     LSC_22Jan93
c        0.  Allowed Ip as well as ne Te Bo to be changed on DoBern Switch
c !!!    1.  Made adjustment to Dql in zones of high damping so Pql Pray 
c            should be pretty much equal.  Key variables: iFudgDmp rFudgDmp
c        2.  Added TORSUPRA coupler option; phaseDeg follows CEN definition
c     LSC_15Jan93
c        0.  Made normalization of parametrized spectrum (DoBram=0) correct.
c        1.  Allowed for Bo to be changed on the DoBern switch
c        2.  Attempted to fix Pray/Pql; by using only discrete n_parallel
c     LSC_08Jan93 :
c        0.  Tokamak de Varennes and PBXM slow couplers selected by character
c            variable couplers(nGrups)
c        1.  dtdV substituted for dVdt in Raystore cause dt can be 0 in TRANSP
c        2.  Ray data written for RW Harvey & CQL3D  with switch DoHarv
c        3.  Grid for reporting Prf and Jrf shifted back to be consistent 
c            with comments.  Thanks to Ted.
c        4.  Camera graphs buffer flushing worked on.  Seems ok now.
c        5.  Parameters not graphed if DoTRAN .eq. 1 as requested by TRANSP
c     LSC_13Nov92 :
c        0.  copy of 7Nov TSC code for J/E iteration in jrf.F as comments
c        1.  fix counting in Giruzzi graphs
c     LSC_07Nov92 : 
c        0.  SMALL set to 1.5e-5 in fe.F for TRANSP 
c        1.  x ray camera w absorbers in XrayCam2 by Valeo, Enright, Ignat
c        2.  last closed flux surface R_, Z_" lcfs " min/max known for graphs;
c        3.  dJ/dE passed to TSC/TRANSP after E/J dJ/dE ;
c        4.  Effective date written with parameters and inputs
c
c      SUBROUTINE LSC(LhPwrMWx,
c     ^               nTSCwrix, nTSCreax, nTSCgrax, nLSCcomx, nTSCunux,
c     ^               nTSCscrx,
c     ^               iRayTrsx, iPlotsix, iErrorx)
c                                       IMPLICIT NONE or equivalent
c                                       IMPLICIT UNDEFINED (A-Z)
c                                       is in file Implic.inc
#include "Implic.inc"
#include "tscunits.inc"
#include "Doflags.inc"
#include "Escan.inc"
      CHARACTER*8 LSCdate
      COMMON /LSCversn/ LSCdate
      CHARACTER*70 ErrMsg
      INTEGER     nTSCwrix, nTSCreax, nTSCscrx, nTSCgrax, nLSCcomx, 
     ^            nTSCunux, iRayTrsx, iPlotsix, iErrorx
      REAL        LhPwrMWx
      REAL    PwrFnd, JrfFnd, PqlFnd
      INTEGER ifirst, iDoRay
c     LhPwrMWx       Lower Hybrid power in MW passed from TSC
c     nTSCwrit       Unit number to which TSC writes equilibrium data
c     nTSCread       Unit number from which TSC reads Prf, Jrf , etc
c     nTSCscrn       Unit number TSC uses for writing to the screen
c     nTSCgraf       Unit number to which LSC writes gnuplot stuff
c                      for each graph
c     nLSCcomm       Unit number to which LSC sends 'screen' output
c     nTSCunused     Unit number not used by TSC; used internally by LSC
c     iRayTrsi     0 use old ray data, and old f_e(v);
c                    use new E_dc for the current
c	           1 calculate new rays, and f_e(v), from new equilibrium
c	           2 use old ray data, but calculate new f_e(v) 
c                    taking account of new n_e and T_e,
c                    use new E_dc for the current
c
c     iPlotsig     0 do not make plots; 1 make plot files asked in input.lhh
c     iError       0 LSC finishes without errors; 1 (or more) errors found
c                 -1 LSC found an error; calling program can keep going
c
c     Confusion posible over the suffix Vec vec and Ary ary; explained below:
c     Vec as in PsiVec ... is on the TSC grid, not on the LSC grid: that is Ary
c     Ary as in PsiAry ... in on the LSC grid, not on the TSC grid: that is Vec
c
#include "params.inc"
#include "RayBins.inc"
#include "RayWrk.inc"
#include "PlPr.inc"
c     -                                 iLastRy is the index of the last ray
c     -                                 traced; or -1 if no rays have been 
c     -                                 traced.  iLastRy controls iDoRay
c                                       iDoRay is a flag in DoRay
c                                       if 1 thru nrays then trace that ray 
c                                       if otherwise, then trace all rays
c

      DATA ifirst /1/
c     -                                 Copy all the variables and flags 
c     -                                 given by TSC into a common block
c     -                                 except iError which is always set to 0.
c     -                                 (value passed from TSC ignored)
c     LSCdate  = 'ddmmmyy '             LSC_date
      LSCdate  = '19Nov00 '
      TotPwr   =  LhPwrMWx * 1.0e+06
      nTSCwrit =  nTSCwrix
      nTSCread =  nTSCreax 
      nTSCscrn =  nTSCscrx
      nTSCgraf =  nTSCgrax
      nLSCcomm =  nLSCcomx
      nLSCcom2 =  nTSCscrx
      nTSCunus =  nTSCunux
      iRayTrsi =  iRayTrsx
      iPlotsig =  iPlotsix
      iError   =  0
      iErrorx  =  0

c     -                                 If this is the first call,
c     -                                 force things a certain way, such as
c     -                                 make sure the rays are traced, and set
c     -                                 the TSC flag to 1 accordingly
c
c     -                                 Clear warning counter; iEdc flag
      call LSCclear
      call RdVars
           if (iError .gt. 0) then
              call LSCtrace ( ' RdVars')
              iErrorx = iError
              return
           endif
c
      if (Do0Edc .ne. 0) then
         iEdc   = 1
         EdcInp = 0.00
c     .                                 set the iEdc switch which was installed
c     .                                 long ago for debugging the sign of
c     .                                 current driven etc; and set the value
c     .                                 of the EdcInp to 0.00
      endif
c
      call RdTSC
           if (iError .gt. 0) then
              call LSCtrace ( ' RdTSC')
              iErrorx = iError
              return
           endif
c
      if(iRayTrsi .eq. 0 .and. ifirst .eq. 1) then 
        iRayTrsi =  2
      endif
c     .                                 if iRayTrsi=0 and firstcall, then
c     .                                 make iRayTrsi=2 so we find f_e etc

      if(iPlotsig .eq. 0) call NoPlots
c

      if(iRayTrsi .ne. 0 .or. ifirst .eq. 1)  then
        ifirst = 0
        call ValInit
c                                       initialization routines 
c                                       for quasilinear calculations 
c                                       -- needed after
c                                       each TSC read
        if (iError .gt. 0) then
          call LSCtrace ( ' ValInit')
          iErrorx = iError
          return
        endif
      endif
c
        if(iRayTrsi .eq. 1 .or. iRayTrsi .eq. 2)  then
          call SpecInit
c     .                                 generate launch spectrum with
c     .                                 nGrps humps at centers(nGrps) with
c     .                                 widths(nGrps) & relative powers(nGrps).
c     .                                 But if nGrps = 0, then get a Brambilla
c     .                                 spectrum using the JE Stevens code.
             if (iError .gt. 0) then
                call LSCtrace ( ' SpecInit')
                iErrorx = iError
                return
             endif
        endif
c
c
c
      if (iRayTrsi .ne. 0) then 
        call PowrInit
c     .                                 apply the launch spectrum to the
c     .                                 total power.
           if (iError .gt. 0) then
              call LSCtrace ( ' PowrInit')
              iErrorx = iError
              return
           endif
c
c       call RyZnInit     !! 9sep 93
c                                       initialization routines
c                                       for ray calculations
c          if (iError .gt. 0) then       !! 9sep 93
c             call LSCtrace (' RyZnInit') !! 9sep 93
c             iErrorx = iError !! 9sep 93
c             return !! 9sep 93
c          endif !! 9sep 93
      endif
c
      if(iRayTrsi .eq. 1 .and. 
     ^     Do1Rpr .eq. 0)         then
              iDoRay = nrays+1
         call DoRay (iDoRay)
           if (iError .gt. 0) then
              call LSCtrace (' DoRay')
              iErrorx = iError
              return
           endif
         call WrRay
           if (iError .gt. 0) then
              call LSCtrace (' WrRay')
              iErrorx = iError
              return
           endif
      endif
c
      if(iRayTrsi .ne. 1 .and. 
     ^     Do1Rpr .eq. 0)         then
         call RdRay
           if (iError .gt. 0) then
              call LSCtrace (' RdRay')
              iErrorx = iError
              return
           endif
      endif
c
      if(iRayTrsi .eq. 1 .and. 
     ^     Do1Rpr .ne. 0)         then
         iDoRay = iLastRy + 1
         if (iDoRay .gt. nrays) iDoRay=1
         call DoRay (iDoRay)
         iLastRy = iDoRay
           if (iError .gt. 0) then
              call LSCtrace (' DoRay')
              iErrorx = iError
              return
           endif
           if(iLastRy .eq. nrays .or. 
     ^        iLastRy .eq.     0      ) then
             call WrRay
             if (iError .gt. 0) then
               call LSCtrace (' WrRay')
               iErrorx = iError
                return
             endif
           endif
      endif

c      if(iRayTrsi .ne. 1 .and.          comment out because of other 
c     ^     Do1Rpr .ne. 0)         then  initializations
c         call RdRay
c           if (iError .gt. 0) then
c              call LSCtrace (' RdRay')
c              iErrorx = iError
c              return
c           endif
c      endif

c                                       trace/read ray trajectories,
c                                       polarization
c     call ryzcent
c                                       compute zone center ray indices
c                                       as required for power deposition
      if (iRayTrsi .ne. 0) then

        call RampPwr        
           if (iError .gt. 0) then
              call LSCtrace (' RampPwr')
              iErrorx = iError
              return
           endif
c                                       ramp up power
c
        call PdepCalc
c                                       compute Prf
      endif

      call JdepCalc
          call JrfDiagn
c         BERNABEI request for 1994 APS Meeting in Minneapolis 
c                                       compute (and plot) jrf
           if (iError .gt. 0) then
              call LSCtrace(' JdepCalc')
              iErrorx = iError
              return
           endif
      call JsplCalc
c                                       compute Js at E_cd pl(us) dE_dc
           if (iError .gt. 0) then
              call LSCtrace(' JsplCalc')
              iErrorx = iError
              return
           endif

      call SmoJandP(Rmax-Rmaj)
c
      call Wr2TSC(PwrFnd, JrfFnd, PqlFnd)
c                                       write deposition profile 
c                                       and current drive profile
c                                       and dJ/dE profile to TSC
c
           if (abs(TotPwr-PwrFnd) .ge. 0.2*TotPwr .or. 
     ^         abs(PqlFnd-PwrFnd) .ge. 0.2*TotPwr )         then
              write(ErrMsg,'('' TotPwr PwrFnd JrfFnd PqlFnd: '',
     ^                            4(1pe10.2) )')
     ^                          TotPwr,PwrFnd,JrfFnd,PqlFnd
              call LSCwarn(ErrMsg)
           endif
c     .
c     .                                 write a warning if power is not
c     .                                 well absorbed or if ray/ql answers
c     .                                 are not too close; give current too
c
           if (iError .gt. 0) then
              call LSCtrace(' Wr2TSC')
              iErrorx = iError
              return
           endif

c      call ProfOut ! no more writing of profiles July 2000

      if(PlFlg(DAMPL) .ge. TRUE) then
         call FirstPas
      endif

      if(PlFlg(DQLFEPL) .ge. TRUE) then
         call Giruzzi
      endif
c
c                                      Inserted, August '92 by Doug P. Enright
c                                      for - Xray bremsstrahlung
c                                      routine
c
      if (DoXcam .eq. 1 .and. iPlotsig .eq. 1) then      
         call xRayCam2
      endif   

           if (iEndRy .gt. 0) then
              call LSCwarn ( ' ray stopped early; see neg rc')
              iErrorx =-iEndRy
              return
           endif

      return
c
      ENTRY PutLSCrestart
      call WrRay
      return
c      
      ENTRY GetLSCrestart
      iError = 0 
      call RdRay
      if (iError .gt. 0 ) then
        call LSCwarn(' no LSC restart file; use dead start')
        return
      endif
      ifirst = 0
      return

      END
c                                                                      |
c                                                                      |
c     AcDc main control module ends         ---------------------------|

      
c 
c     block data programs                   ---------------------------|
c                                                                      |
c                                                                      |
      BLOCK DATA AcDcBk
c
#include "Implic.inc" 
#include "params.inc"
c#include "xparams.inc"
#include "DqlBins.inc"
#include "PIetc.inc"
      DATA    PI    , TWOPI , RTPI  , PIO4  , R3O2  , R1O2    /
     ^        3.1416, 6.2832, 1.77  ,0.7854 ,1.22475,0.707107 /
 
#include "MKSetc.inc"
      DATA    CLIGHT, ECOULB, ELECMS, PROTMS /
     ^        2.9979, 1.6022, 9.1095, 1.6726 /
c            10^8 m/s 10^-19c 10^-28g 10^-24g
 
#include "CGSetc.inc"
#include "FeBins.inc"
c     .                                 iSMO = 2 --> Do smoothing as
c     .                                 EValeo did in original code
c     .                                 using smoothed Dql=Dql(iv,ip,2)
c     .                                 iSMO = 1 --> 
c     .                                        Use unsmoothed Dql(.,.,1)
c     .                                 and smoothed dfdv(iv,ip) for 
c                                              Pql & Jrf
c                smooth Dql is ALWAYS used in calculating fe(iv,ip,iITR)
c
      DATA    CLIcgs, ECHARG, EMASS,   TeUnits /
     ^        3.0e10,4.8e-10, 9.1e-28, 1000. /
      DATA    iSMO,      nv /
     ^           1, NVELDIM /
 
#include "RayBins.inc"
#include "RayWrk.inc"
      DATA    iLastRy / -1 /
      DATA    lfast, nfreq, npsi, nstep / 
     ^          0,   100, NPSIDIM,20000 /
      DATA      nrays,   ntors,   npols /
     ^        NRAYDIM, NTORDIM,       1 /
      DATA    nzones  /
     ^        NZONDIM /
      DATA    PhaseDeg(1), PhaseDeg(2), PhaseDeg(3), nslices / 
     ^                90.,        180.,        135.,     301 /
      DATA    couplers(1), couplers(2), couplers(3)          /
     ^        'PBXMFAST' , 'PBXMSLOW' , 'TOKDEVAR'           / 
c
#include "Ramppwr.inc"
      DATA     nrampup,    nflat /
     ^        NRAMPDIM, NFLATDEF /
#include "Jrf.inc"
      DATA    vnmax, dEdcAmnt / 0.99 , 0.0001 /
      DATA    DiffuJrf, PrfSpred / 0.000, 0.000 /
c
      DATA    enpol,  fghz, HstpLH /
     ^           0.,   4.6, .005   /
      DATA    nparmax, nparmin, thet0, ScatKdeg, TurnNegs /
     ^           5.5 ,     2.5,   0.0,     0.00,       0  /
      DATA    npolmax, npolmin /
     ^           +1.0,    -1.0 /
      DATA    nGrps  , centers(1), centers(2), centers(3) /
     ^           3   ,      4.0  ,      4.0  ,      4.0   /
      DATA             widths(1) , widths(2) , widths(3)  /
     ^                      1.0  ,      1.0  ,      1.0   /
      DATA             powers(1) , powers(2) , powers(3)  /
     ^                      1.0  ,      0.1  ,      0.1   /
#include "PlPr.inc"
      DATA 
     ^     RAYPL,    SPECPL,   RFDPL,    RFDPSPL,  DAMPL,    JRFPL,    
     ^     PITPRFPL, DQLFEPL
     ^     /
     ^     1,        2,        3,        4,        5,        6,
     ^     7,        8
     ^     /

      DATA PlFlg      /
     ^     nPlFlg * 0 /
      DATA 
     ^     RAYWR,    FSTFRCWR, NPAPWRWR
     ^     /
     ^     1,        2,        3
     ^     /
      DATA PrFlg      /
     ^     nPrFlg * 0 /

      DATA idiag(1),idiag(2),idiag(3),idiag(4),idiag(5) /
     ^     NRAMPDIM,      0 ,      0 ,      0 ,      0  / 
     
      DATA idiag(6),idiag(7),idiag(8),idiag(9),idiag(10)/  
     ^           0 ,      0 ,      0 ,      0 ,      0  / 
#include "Doflags.inc"
      DATA DoBram, DoTRAN /
     ^          1,      0 /
      DATA DoXcam, Do1Rpr, Do0Edc /
     ^          0,      0,      0 /
      DATA FeWind               /
     ^     1, NVELDIM, 1, NPSIDIM /
      DATA DqlWind              /
     ^     1, NVELDIM, 1, NPSIDIM /
      DATA   nsmoo,    nsmw /
     ^     NSMODEF, NSMWDEF /
      DATA Vmin, Vmax, WeghtItr /
     ^      -1.,  1. ,     0.20 /
#include "Escan.inc"
      DATA iEdc, EdcInp / 0, 0.001 /

      END
c                                                                      |
c                                                                      |
c     block data program                    ---------------------------|

c     RdVars    Read Variables, sets constants ------------------------|
c                                                                      |
c                                                                      |
      SUBROUTINE RdVars
#include "Implic.inc"
#include "tscunits.inc"
#include "params.inc"
c#include "xparams.inc"
#include "DqlBins.inc"
#include "FeBins.inc"
#include "RayBins.inc"
#include "CGSetc.inc"
#include "MKSetc.inc"
#include "PIetc.inc"
#include "ProfBody.inc"
#include "PlPr.inc"
#include "Doflags.inc"
#include "RayWrk.inc"
#include "Ramppwr.inc"
#include "Jrf.inc"
c#include "Xray.inc"
      CHARACTER*12 FileNa1
      INTEGER ifirst,i
      REAL    tmpdum
      DATA    ifirst /1/
      DATA    FileNa1 /'input.lhh'/
c
c     NAMELIST /inpvalue/  goes here from the include file:
c     NAMELIST /inpexprt/  goes here from the include file:
#include "Inpval.inc"
c     .                                 Reset nrays,ntors,npols so that
c     .                                 the error checker can detect whether
c     .                                 ntors&npols are basic inputs or
c     .                                 nrays is basic input.  This for back-
c     .                                 compatibility with input files.
      if (DoTRAN .ne. TRUE) then
         nrays = -1
         ntors = -1
         npols = -1
         TailPeps = 0.00
         TailTeps = 0.00
         TailNeps = 0.00
      endif
c     .
      if (DoTRAN .ne. TRUE) then
         open (nTSCunus,status='old',file = FileNa1, err=1300 )
         read (nTSCunus, inpvalue)
         read (nTSCunus, inpexprt)
         close (nTSCunus)
      endif

 12   begin = 0.
      
      woc2 = (TWOPI * fghz / CLIGHT)**2 * 1.e2
      woc4 =  woc2**2
      omega=  TWOPI*fghz * 1.e9
 15   continue
c
      if (DiffuJrf .lt. 0.0000) then
              DiffuJrf = 0.00
              call LSCwarn(' DiffuJrf set to 0')
              endif
      if (PrfSpred .lt. 0.0000 .or.
     ^   (DiffuJrf .eq. 0.0000 .and.
     ^    PrfSpred .gt. 0.0000))then
              PrfSpred = 0.00
              call LSCwarn(' PrfSpred set to 0')
              endif
      if (PrfSpred .gt. 1.0000) then
              PrfSpred = 1.00
              call LSCwarn(' PrfSpred set to 1')
              endif
      if (nparmin .ge. nparmax .and. DoBram .eq. 0) then
              tmpdum = nparmax
              nparmax = nparmin+0.001
              nparmin = tmpdum -0.001
              call LSCwarn(' nparmin/max reversed')
              endif
c     Check dimensions now
      if ( npsi   .gt. NPSIDIM   ) then
              npsi   = NPSIDIM
              call LSCwarn ( ' npsi set to NSPIDIM ')
              endif
c
c     .                                 The if-else-if sequence below
c     .                                 is to maintain compatibility w.
c     .                                 old usage npols=1,ntors=nrays,
c     .                                 ntors not explicitly used;
c     .                                 IF ntors&nrays not given then
c     .                                 use NTORDIM

      if ( ntors  .gt. NTORDIM   ) then
              ntors  = NTORDIM
              call LSCwarn ( ' ntors set to NTORDIM ')
              endif
      if ( npols  .gt. NPOLDIM   ) then
              npols  = NPOLDIM
              call LSCwarn ( ' npols set to NPOLDIM ')
              endif
      if ( nrays  .gt. NRAYDIM   ) then
              nrays  = NRAYDIM
              call LSCwarn ( ' nrays set to NRAYDIM ')
              endif
      if ( npols  .lt. 1         ) then
              npols  = 1
              endif
      if ( nrays .eq. -1 .and. ntors .eq. -1 ) then
                  npols = 1
                  ntors = NTORDIM
                  nrays = NTORDIM
c     .                                 IF ntors given calculate nrays
        else if ( nrays  .eq. -1 ) then 
                  nrays  = ntors*npols
c     .                                 IF nrays given calculate ntors
        else if ( ntors .eq. -1)    then 
                  ntors  = nrays/npols
                  if ( ntors  .gt. NTORDIM   ) then
                     ntors  = NTORDIM
                     nrays  = ntors*npols
                     call LSCwarn ( ' ntors set to NTORDIM ')
                  endif
      endif
c
      if (nrays .ne. ntors*npols) then
                  nrays = ntors*npols
                  call LSCwarn ( ' nrays set to ntors*npols ')
              endif
c
      if ( nv     .gt. NVELDIM     ) then 
              nv     = NVELDIM
              call LSCwarn ( ' nv set to NVELDIM ')
              endif
      if ( nzones .gt. NZONDIM   ) then
              nzones = NZONDIM
              call LSCwarn ( ' nzones set to NZONDIM ')
              endif
      if ( nRampUp  .gt. NRAMPDIM ) then
              nRampUp = NRAMPDIM
              call LSCwarn ( ' nRampUp set to NRAMPDIM ')
              endif
      if ( nsmoo  .gt. nv/3   .or.
     ^     nsmoo  .gt. nzones/3    ) then
                call LSCwarn (' nsmoo set to min(nv,nzones)/3 ')
                nsmoo = min0(nv, nzones) / 3
                endif
c                                       The relative size of these dimensions
c                                       is implicitly assumed in work arrays.
c     if ( NVELDIM .lt. (NPSIDIM + NZONDIM)) then           ! NWKVDIM avoids
c               call LSCstop (' NVELDIM < NPSIDIM+NZONDIM' )! this clumsiness
c               endif                             ! =NVELDIM+NPSIDIM+NZONDIM
c
      if(mod(nsmoo, 2) .eq. 0) then
                call LSCwarn (' nsmoo MUST BE ODD ')
                nsmoo = iabs(nsmoo - 1)
                endif
      if ( nsmw .lt. nsmoo/8 ) then
                call LSCwarn (' nsmoo-width seems too SMALL ')
                endif
      if ( nsmw .ge. nsmoo   ) then
                call LSCwarn (' nsmoo-width seems too LARGE ')
                endif
      if ( WeghtItr .gt. 1.0 .or. WeghtItr .lt. 0.0 ) then  
                WeghtItr = 0.5
                call LSCwarn (' WeghtItr set to 0.50 ')
                endif
      if ( TailTeps .gt. 0.00 .and. TailNeps .gt. 0.00 ) then
        TailPeps = TailNeps/TailTeps
      else if (TailTeps .gt. 0.00 .and. TailPeps .gt. 0.00 ) then
        TailNeps = TailPeps*TailTeps
      else if (TailNeps .gt. 0.00 .and. TailPeps .gt. 0.00 ) then
        TailTeps = TailNeps/TailPeps
      else
        TailTeps = 0.00
        TailNeps = 0.00
        TailPeps = 0.00
      endif
      if (TailTeps .gt. 0.3 .or.
     ^    TailPeps .gt. 0.3 .or.
     ^    TailNeps .gt. 0.1      ) then
        call LSCwarn(' Too much fast electron tail')
        TailTeps = 0.00
        TailNeps = 0.00
        TailPeps = 0.00
      endif
      if ( nGrps .gt. NGRPDIM .or. nGrps .le. 0 ) then
                nGrps = 1
                call LSCwarn (' nGrps being set to 1 ')
                endif
      do i=1,nGrps
         if(powers(i) .le. 0.00) then
            powers(i) = 0.001
            call LSCwarn( ' powers(i) being set to 0.001 ')
         endif
      enddo
                 
      if (DoBram .eq. 1 ) then                
        do i=1,nGrps

          if (  
     ^ couplers(i).ne.'PBXMFAST'.and.couplers(i).ne.'PBXMSLOW'.and.
     ^ couplers(i).ne.'SLOWSLOW'.and.couplers(i).ne.'TORSUPRA'.and.
     ^ couplers(i).ne.'TOKDEVAR'.and.couplers(i).ne.'TFTRLHCD'.and.
     ^ couplers(i).ne.'JET_LHCD'.and.couplers(i).ne.'USRSPEC1'.and.
     ^ couplers(i).ne.'USRSPEC2'.and.couplers(i).ne.'USRSPEC3')then
              couplers(i)    = 'PBXMSLOW'
              call LSCwarn (' couplers(i) being set to PBXMSLOW ')
          endif

        enddo
      endif

      if(ifirst .eq. 1) then
         ifirst= 0
         PIO4  = atan(1.)
         PI    = 4. * PIO4
         TWOPI = 2. * PI  
         RTPI  = sqrt(PI)
         dompesqdn = 4. * PI * (ECHARG/1.e-10)**2 / (EMASS/1.e-28) *
     ^                                                        1.e8
         RESTenergy = (EMASS/1.e-28)*(CLIcgs/1.e10)**2 *1.e-8
         ERGperEV = 1.6e-12
         keVperERG = 1. / (1000. * ERGperEV)
         if (DoTRAN .eq. 0 ) then
           call GrafParm
         endif
      endif


      return
 1300 call LSCstop (' LSC cant find input.lhh ')
      return
      END
c     -                                                                |
c     RdVars    ends                        ---------------------------|
c     
c     SpecInit begins -------------------------------------------------|
c     -                                                                |
c     -                                                                |
      SUBROUTINE SpecInit
#include "Implic.inc"
#include "params.inc"
#include "tscunits.inc"
#include "PlPr.inc"
#include "RayBins.inc"
#include "RayWrk.inc"
#include "Doflags.inc"
      INTEGER oldNtors, oldNpols, oldnGrps, oldNslic, oldDoBram
      REAL
     ^        oldPowrs(NGRPDIM),oldCents(NGRPDIM),oldWidts(NGRPDIM) 
      REAL
     ^        oldPhsDg(NGRPDIM)
      CHARACTER*8 oldCoupl(NGRPDIM)
      CHARACTER*80 pline
      CHARACTER*40 MyString
      INTEGER igrup, i, iry, ity, ipy, ifirst
      INTEGER ixmjr, ixmnr, iymjr, iymnr, nRyPrGr
      REAL
     ^        powtot, RayGroup, GrpNorm(NGRPDIM)
      REAL
     ^        eNs (NTORDIM,NGRPDIM), SofNs (NTORDIM,NGRPDIM)
      REAL
     ^        ens1(NGRPDIM*NTORDIM), SofNs1(NGRPDIM*NTORDIM)
      EQUIVALENCE (  eNs (1,1),   eNs1(1) )
      EQUIVALENCE (SofNs (1,1), SofNs1(1) )
      REAL
     ^        SpShape, MaxOfAry, MaxPwr, smx, xmin, xmax, EXCLUDED
      DATA    EXCLUDED, xmin, xmax, ixmjr,ixmnr,iymjr,iymnr /
     ^           1.20 ,-10.0, 10.0,     2,    5,    2,    1 /
      DATA    ifirst / 1 /
      REAL    ZERO, ONE
      DATA    ZERO, ONE/
     ^         0.0, 1.0/
      EXTERNAL SpShape
c     .                                 If this is not the first call, then
c     .                                 test to see if any of the parameters
c     .                                 of the spectrum have changed.  
c     .                                 If stable, then return to avoid another
c     .                                 calculation, which in brambJES is
c     .                                 time consuming.
      if (ifirst .eq. 0 ) then
        i = 0
        do 5 igrup=1,nGrps
          if( oldPowrs(igrup) .ne. powers  (igrup)) i=i+1
          if( oldCents(igrup) .ne. centers (igrup)) i=i+1
          if( oldWidts(igrup) .ne. widths  (igrup)) i=i+1
          if( oldPhsDg(igrup) .ne. phaseDeg(igrup)) i=i+1
          if( oldCoupl(igrup) .ne. couplers(igrup)) i=i+1          
 5      continue
          if( oldNtors .ne. ntors   .or. oldnGrps .ne. nGrps 
     ^   .or. oldNpols .ne. npols  
     ^   .or. oldNslic .ne. nslices .or. oldDoBram .ne. DoBram )i=i+1
c
        if (i .eq. 0) return
c
        iRayTrsi = 1 
c
        else
        ifirst = 0
      endif
c
c
c     .                                 Make the spectrum in poloidal number
      npol(1) = (npolmax+npolmin)/2.0
      if (npols .gt. 1 )  call ugrid(npol,npols,npolmin,npolmax)


      
c     .                                 IF ----------------------------|  
c     .                                 If we are not to do a Brambilla calc,
c     .                                 make a model spectrum using 
ca    .                                 gaussian form.
                                        if(DoBram .eq.  0 ) then
      call ugridEXC (ntor, ntors, nparmin, nparmax, EXCLUDED)
      powtot = 0.
      
      do 17 igrup = 1, nGrps
        GrpNorm(igrup) = 0.
          do 15  i = 1, ntors
            GrpNorm(igrup) = GrpNorm(igrup) + 
     ^        SpShape(ntor(i), centers(igrup), widths(igrup))
 15       continue
        GrpNorm(igrup) = 1. / GrpNorm(igrup)
 17   continue
c
      do 30 ity = 1, ntors
         Spec(ity) = 0.
         do 25 igrup = 1, nGrps
            RayGroup = GrpNorm(igrup) * powers(igrup) *
     ^        SpShape(ntor(ity), centers(igrup), widths(igrup))
            Spec(ity) = Spec(ity) + RayGroup
 25      continue
 30   continue
c
      call vecnorm(ntors,Spec)

c
c     .                                 But if DoBram =1, then
c     .                                 get a Brambillla spectrum.
c     .                                 ELSE --------------------------|
                                        else if (DoBram .eq. 1) then
c
         
c     .                                 Parcel out a ray count by groups.
c     .                                 Over distribute if a it doesnt come out
c     .                                 even
                  nRyPrGr = ntors / nGrps 
                  if (nGrps * nRyPrGr .lt. ntors) nRyPrgr = nRyPrGr + 1
c     .
      if (nRyPrGr .gt. NTORDIM) nRyPrGr = NTORDIM
      call BrodCast(NTORDIM*NGRPDIM, SofNs , 0.)
      call BrodCast(NTORDIM*NGRPDIM,   eNs , 0.)
      do 50 i=1,nGrps              
        call BrambJES(fghz, phaseDeg(i), nRyPrGr, eNs(1,i), SofNs(1,i),
     ^                      EXCLUDED, nslices, couplers(i), TurnNegs )
 50   continue
      do 55 igrup=1,nGrps
        do 55 ity=1,nRyPrGr
          SofNs(ity,igrup) = powers(igrup)*SofNs(ity,igrup)
 55   continue
      call PikSr2NR(NTORDIM*NGRPDIM, SofNs, eNs)
      do 60 i = 1, ntors
        ntor(i) = eNs1  (i + NTORDIM*NGRPDIM-ntors )
        Spec(i)     = SofNs1(i + NTORDIM*NGRPDIM-ntors )
 60   continue
      call PikSr2NR(ntors, ntor, Spec)
      call vecnorm(ntors,Spec)
                                        endif
c     .                                 ENDIF -------------------------|
c     .
c     .                                 It can happen that there are two
c     .                                 rays at the same n_//. Check and report
      do 80 i = 2, ntors
        if ( ntor(i) .ne. ntor(i-1) ) go to 80
        write(MyString,'(''identical n// at '',1pe10.3)') ntor(i)
        call LSCwarn(MyString)
 80   continue
c
c
c     Option to write out the n-tors and Powers of rays     -----------|
c     -----------------------------------------------------------------|
c     Spectrum IO 
      if ( PrFlg(NPAPWRWR) .ge. TRUE ) then
         write(nLSCcom2, 600)
 600     format(/,'  Ray Index     n-tor  Power(W)')
 601     format(1x,     8x,i2,1x,1pe9.2,1x,1pe9.2  )
         do 160 ity=1,ntors
            write(nLSCcomm, 601) ity, ntor(ity), Spec(ity)
 160     continue
      endif
c     Option ends -------------------------------------------------|
c
c     Option to plot ----------------------------------------------|
       if(PlFlg(SPECPL) .ge. TRUE) then 
        call EZinit
        call GNinit(nTSCgraf)
        MaxPwr = MaxOfAry(Spec,ntors)
        call EZrnd2  (MaxPwr, smx, iymjr, iymnr)
        call ezsets(100,500,125,375,xmin,xmax,ZERO,smx,1)
c       call GNsets(100,500,125,375,xmin,xmax,ZERO,smx,1)
        call GNsets(  0,500,  0,375,xmin,xmax,ZERO,smx,1)
        call ezwrit(100,400,'Relative power vs n_tor$',0,0)
        call GNtitl(        'Relative power vs n_tor'     )
        call ezbars(ntor, Spec, ntors, 'y')
        call GNbars(ntor, Spec, ntors, 'y')
        call ezaxes( ixmjr, ixmnr,  iymjr, iymnr )
        if (npols .eq. 1) then
          write(pline,'(f5.2,'' GHz w/ '',
     ^     i3, '' rays in '', i2,'' groups.$'' )') fghz, ntors, nGrps
        else
          write(pline,'(f5.2,'' GHz;'',
     ^     i3, '' ntors;'', i2, '' nGrps;'',
     ^     i2,'' npols$'' )') fghz, ntors, nGrps, npols
        endif
        call ezwrit(  1,750, pline, 0, 0)
        call GNlist(  1,750, pline, 0, 0)
c
        if(DoBram .eq. 1) then
          write(pline,'(''Brambilla with '', i4,
     ^                 '' slices.$'' )') nslices
          call ezwrit(  1,725, pline, 0, 0)
          call GNlist(  1,725, pline, 0, 0)
          write(pline,'( a8,'' at'', f7.1, 
     ^                      '' deg &'', f4.1,'' pwr$'')')
     ^                    couplers(1),phaseDeg(1),powers(1)
          call ezwrit(  1,700, pline, 0, 0)
          call GNlist(  1,700, pline, 0, 0)
          if (nGrps .ge. 2) then
          write(pline,'( a8,'' at'', f7.1, 
     ^                      '' deg &'', f4.1,'' pwr$'')')
     ^                    couplers(2),phaseDeg(2),powers(2)
          call ezwrit(  1,675, pline, 0, 0)
          call GNlist(  1,675, pline, 0, 0)
          endif
          if (nGrps .ge. 3) then
          write(pline,'( a8,'' at'', f7.1, 
     ^                      '' deg &'', f4.1,'' pwr$'')')
     ^                    couplers(3),phaseDeg(3),powers(3)
          call ezwrit(  1,650, pline, 0, 0)
          call GNlist(  1,650, pline, 0, 0)
          endif

        else
          write(pline,'(
     ^       ''centers: '', 3(f5.2))') (centers (i),i=1,nGrps)
          call DolAtEnd(pline)
          call ezwrit(  1,725, pline, 0, 0)
          call GNlist(  1,725, pline, 0, 0)

          write(pline,'(
     ^       ''widths:  '', 3(f5.2))') (widths  (i),i=1,nGrps)
          call DolAtEnd(pline)
          call ezwrit(  1,700, pline, 0, 0)
          call GNlist(  1,700, pline, 0, 0)

          write(pline,'(
     ^       ''powers:  '', 3(f5.2))') (powers  (i),i=1,nGrps)
          call DolAtEnd(pline)
          call ezwrit(  1,675, pline, 0, 0)
          call GNlist(  1,675, pline, 0, 0)

        endif
        call pSmoUnsm(ntor,Spec,ntors)
        call EZwrit(  1,30,EquTitle (1:50)//'$',0,0)
        call GNlist(  1,30,EquTitle (1:50)//'$',0,0)
        call EZwrit(  1, 5,EquTitle(51:96)//'$',0,0)
        call GNlist(  1, 5,EquTitle(51:96)//'$',0,0)
        call MkGrfLst(' Launched lwr hybrid spectrum ')

        call EZfini(0,0)
        call GNfini

      endif
c dwi save        open (nTSCunus,status='old',file = FileNa1, err=1300 )
c     Option to plot ends -----------------------------------------|
c
c     Prepare to return by saving old values
      do 90 igrup = 1,nGrps
      oldPowrs(igrup) = powers(igrup)
      oldCents(igrup) = centers(igrup)
      oldWidts(igrup) = widths(igrup)
      oldPhsDg(igrup) = phaseDeg(igrup)
      oldCoupl(igrup) = couplers(igrup)
 90   continue
c     ntors    = nInclIBW !
      nrays    = ntors*npols
      oldNtors = ntors
      oldNpols = npols
      oldnGrps = nGrps
      oldNslic = Nslices
      oldDoBram= DoBram
c
c
      return        
c
c     -----------------------------------------------------------------|
      ENTRY PowrInit
      iry=1
      do 100 ity=1,ntors
      do 100 ipy=1,npols
         npar (1,iry) = ntor(ity)
         iry = iry+1
 100  continue
      call rspwr(1.)
c     generate launch spectrum at full power
      do 110 iry = 1, nrays
        call blockcpy(nzones - 1, power(2, iry), 1, power(1, iry), 1)
c       copy 
 110   continue
c     copy results onto entire trajectory 
c
c
      return
      END
c
      REAL FUNCTION SpShape(ParValue, center, width)
#include "Implic.inc"
      REAL ParValue, center, width, relnpar, exparg
      relnpar = (ParValue - center) / width
      exparg = 0.5 * relnpar * relnpar
      SpShape = exp( - exparg)
      return
      END
c     -----------------------------------------------------------------|
      SUBROUTINE DolAtEnd (SomeString)
      CHARACTER*(*) SomeString
      CHARACTER*1 DOLLAR
      INTEGER i,lastch, length
      DATA DOLLAR /'$'/
      length = len(SomeString)
      do 10 i=length,1,-1
	lastch=i
        if (SomeString(i:i) .ne. ' ') go to 11
 10   continue
      SomeString(1:1) = DOLLAR
      return
c
c
 11   continue
      lastch=lastch+1
      if (lastch .gt. length) lastch=length
      SomeString(lastch:lastch) = DOLLAR
      return
      END
c     -----------------------------------------------------------------|
      SUBROUTINE pSmoUnsm(xNpar,ySpec,n)
#include "Implic.inc"
#include "params.inc"
#include "DqlBins.inc"
#include "FeBins.inc"
#include "WkAry.inc"
      CHARACTER*10 MyString
      INTEGER n, i, iv, ivtabl, iymjr, iymnr, nsm2
      EXTERNAL ivtabl
      REAL    xNpar(n), ySpec(n), ymax, vbig
      REAL    PofV(NVELDIM) 
c     REAL    wkv (NVELDIM) ! in WkAry.inc
      REAL    ZERO, ONE
      DATA    ZERO, ONE/
     ^         0.0, 1.0/
      call BrodCast(nv,PofV,0.00)
      call BrodCast(nv,wkv ,0.00)
      nsm2 = (nsmoo - 1) / 2
      ymax = 0.00
      vbig = 0.50
      do 10 i=1,n
        iv = ivtabl(xNpar(i))
        wkv (iv) = wkv(iv) + ySpec(i)
        if (ymax .lt. wkv(iv)) ymax = wkv(iv) 
        if (vbig .lt. abs(vpar(iv))) vbig = 1.00
 10   continue
      call EZrnd2  (ymax, ymax, iymjr, iymnr)
      call convolve(nv,nsm2,qlsm,PofV,wkv)
      call EZsets(600,1000,475,725, -vbig,vbig, ZERO, ymax,1)
c     call GNsets(600,1000,475,725, -vbig,vbig, ZERO, ymax,1)
      call GNsets(500,1000,375,750, -vbig,vbig, ZERO, ymax,1)
      call ezwrit(600,750,'Relative power vs v/c$',0,0)
      call GNtitl(        'Relative power vs v/c '    )
      call EZaxes(2,5,iymjr,iymnr)
      do 20 iv=1,nv
        if (wkv(iv) .gt. 0.00) call EZbars(vpar(iv),wkv(iv),1,'y')
 20   continue
                               call GNbars(vpar    ,wkv   ,nv,'y')
      call EZsets(600,1000,125,375, -vbig,vbig, ZERO, ymax,1)
c     call GNsets(600,1000,125,375, -vbig,vbig, ZERO, ymax,1)
      call GNsets(500,1000,  0,375, -vbig,vbig, ZERO, ymax,1)
      call ezwrit(600,400,'Relative power smoothed$',0,0)
      call GNtitl(        'Relative power smoothed'     )
      call EZaxes(2,5,iymjr,iymnr)
      call EZcurv(vpar,PofV,nv)
      call GNcurv(vpar,PofV,nv)
      write(MyString,'(''nsm2 '',i3,''$'')') nsm2
      call ezwrit(850,350,MyString,0,0)
      write(MyString,'(''nv/2 '',i3,''$'')') (nv-1)/2
      call ezwrit(850,325,MyString,0,0)
      return
      END
c     -----------------------------------------------------------------|
c     -                                                                |
c     -                                                                |
c     SpecInit ends   -------------------------------------------------|
c      
c     ValInit begins  -------------------------------------------------|
c     -                                                                |
c     -                                                                |
c
      SUBROUTINE ValInit
#include "Implic.inc"
#include "tscunits.inc"
      call MiscInit                     
c                                       doesn't fit elsewhere
      call MkGrids
c                                       set up velocity, psi grids 
c                                       needed for quasilinear calculations
c     call WrGrids
c                                       output same ! not after June 2000
      call ProfInit
c                                       interpolate Ne, Ni, Te, Ti 
c                                       to PsiAry from TSC data points
      call FeInit
c                                       compute constants needed for 
c                                       solution of Fe ....
      call DqlInit
c                                       and for solution of Dql
c     call WrDcoll                    
c                                       output same ! not after June 2000
      call DqlClear
c                                       set Dql == 0
      return
      END
c
c                                                                      |
c     Openings, Initializations end         ---------------------------|
c
cc 
c     LowOf1(ch); LSCPause; MaxOfAry; MinOfAry ------------------------|
c                                                                      |
c                                                                      |
      SUBROUTINE LowOf1(ch)
#include "Implic.inc"
      CHARACTER*1 ch
      INTEGER int
      int = ichar(ch)
      if (int .le. 90 .and. int .ge. 65) int = int + 32
      ch = char(int)
      return
      END
c     ------------------------------------------------------------------
      REAL FUNCTION MaxOfAry(array, n)
#include "Implic.inc"
      INTEGER n, i
      REAL    array(n), tmp
      tmp = array(1)
      do 10 i = 2, n
        if(array(i) .gt. tmp) tmp = array(i)
 10   continue
      MaxOfAry = tmp
      return
      END
c     ------------------------------------------------------------------
      REAL FUNCTION MinOfAry(array, n)
#include "Implic.inc"
      INTEGER n, i
      REAL    array(n), tmp
      tmp = array(1)
      do 10 i = 2, n
        if(array(i) .lt. tmp) tmp = array(i)
 10   continue
      MinOfAry = tmp
      return
      END
c                                                                      |
c                                                                      |
c     end Low Of 1 ch; MaxOfAry; MinOfAry -----------------------------|
c     ------------------------------------------------------------------
c      SUBROUTINE LSCPause
c#include "Implic.inc"
c#include "Doflags.inc"
c#include "params.inc"
c#include "tscunits.inc"
c      CHARACTER*1 ch
c      if (DoPaus .ge. TRUE) then
c        write(nTSCscrn,'('' ... hit <cr> to continue ...'')')
c        read (5,'(a1)') ch
c      endif
c      return
c      END
c
      SUBROUTINE GrafParm
#include "Implic.inc"
#include "params.inc"
#include "Doflags.inc"
#include "DqlBins.inc"
#include "Jrf.inc"
#include "FeBins.inc"
#include "Ramppwr.inc"
#include "RayBins.inc"
#include "RayWrk.inc"
#include "PlPr.inc"
      call GrafFixP('BEGIN PLOT', 0    )
      call GrafFltP(' fghz     ', fghz)
      call GrafFltP(' HstpLH   ', HstpLH)
      call GrafFixP(' nstep    ', nstep)
      call GrafFixP(' nrays    ', nrays)
      call GrafFixP(' ntors    ', ntors)
      call GrafFixP(' npols    ', npols)
      call GrafFixP(' nGrps    ', nGrps)
      call GrafFixP(' npsi     ', npsi)
      call GrafFixP(' nzones   ', nzones)
      call GrafFixP(' nv       ', nv)
      call GrafFixP(' nsmoo    ', nsmoo)
      call GrafFixP(' nsmw     ', nsmw)
      call GrafFixP(' nRampUp  ', nRampUp)
      call GrafFixP(' nFlat    ', nFlat)
      call GrafFixP(' idiag(1) ', idiag(1))
      call GrafFixP(' idiag(2) ', idiag(2))
      call GrafFltP(' WeghtItr ', WeghtItr)
      call GrafFltP(' ScatKdeg ', ScatKdeg)
      call GrafFltP(' DiffuJrf ', DiffuJrf)
      call GrafFltP(' PrfSpred ', PrfSpred)
      call GrafFixP(' DoTRAN   ', DoTRAN)
      call GrafFixP(' DoXcam   ', DoXcam)
      call GrafFixP(' Do1Rpr   ', Do1Rpr)
      call GrafFixP(' Do0Edc   ', Do0Edc)
c
      if ( DoBram .eq. 0 ) then
      call GrafFixP(' DoBram   ', DoBram)
      call GrafFltP(' nparmax  ', nparmax)
      call GrafFltP(' nparmin  ', nparmin)
      call GrafFltP(' npolmax  ', npolmax)
      call GrafFltP(' npolmin  ', npolmin)
      call GrafFixP(' nGrps    ', nGrps  )
      call GrafFltP(' centers-1', centers(1))
      call GrafFltP(' widths-1 ', widths (1))
      call GrafFltP(' powers-1 ', powers (1))
      if ( nGrps .ge. 2 ) then
      call GrafFltP(' centers-2', centers(2))
      call GrafFltP(' widths-2 ', widths (2))
      call GrafFltP(' powers-2 ', powers (2))
      endif
      if ( nGrps .ge. 3 ) then
      call GrafFltP(' centers-3', centers(3))
      call GrafFltP(' widths-3 ', widths (3))
      call GrafFltP(' powers-3 ', powers (3))
      endif
      endif
c
      if ( DoBram .eq. 1 ) then
      call GrafFixP(' DoBram   ', DoBram)
      call GrafFixP(' nslices  ', nslices)
      call GrafFixP(' nGrps    ', nGrps  )
      call GrafChrS(' couplers1', couplers(1))
      call GrafFltP(' paseDeg-1', phaseDeg(1))
      call GrafFltP(' powers-1 ', powers (1))
      if ( nGrps .ge. 2 ) then
      call GrafChrS(' couplers2', couplers(2))
      call GrafFltP(' paseDeg-2', phaseDeg(2))
      call GrafFltP(' powers-2 ', powers (2))
      endif
      if ( nGrps .ge. 3 ) then
      call GrafChrS(' couplers3', couplers(3))
      call GrafFltP(' paseDeg-3', phaseDeg(3))
      call GrafFltP(' powers-3 ', powers (3))
      endif
      endif
c
      call GrafFixP(' TurnNegs ', TurnNegs)
      if (TailTeps*TailPeps*TailNeps .gt. 0.00) then
      call GrafFltP(' TailTeps ', TailTeps)
      call GrafFltP(' TailPeps ', TailPeps)
      call GrafFltP(' TailNeps ', TailNeps)
      endif
      call GrafFltP(' Vmin     ', Vmin)
      call GrafFltP(' Vmax     ', Vmax)
      call GrafFixP(' nfreq    ', nfreq)
      call GrafFixP(' lfast    ', lfast)
c     call GrafFltP(' thet0    ', thet0)
      call GrafFltP(' thet0unus', thet0)
c
      call GrafFixP(' PPSI     ', PPSI)
      call GrafFixP(' PNX      ', PNX )
      call GrafFixP(' PNZ      ', PNZ)
      call GrafFixP(' PIMP     ', PIMP )
      call GrafFixP(' PWORDS   ', PWORDS)
      call GrafFixP(' NPSIDIM  ', NPSIDIM)
      call GrafFixP(' NRAYDIM  ', NRAYDIM)
      call GrafFixP(' NTORDIM  ', NTORDIM)
      call GrafFixP(' NPOLDIM  ', NPOLDIM)
      call GrafFixP(' NVELDIM  ', NVELDIM)
      call GrafFixP(' NZONDIM  ', NZONDIM)
      call GrafFixP(' NPLTDIM  ', NPLTDIM)
      call GrafFixP(' NGRPDIM  ', NGRPDIM)

      call MkGrfLst(' List of LSC inputs, params   ')
      call GrafFixP('END PLOT  ', 0   )
      return 
      END

      SUBROUTINE GrafFixP ( MyString, Iargumnt )
#include "Implic.inc"
#include "tscunits.inc"
c     SUBROUTINE GrafFltP ( MyString, Rargumnt )  ! given as ENTRY !
c     SUBROUTINE GrafChrS ( MyString, Cargumnt )  ! given as ENTRY !
      CHARACTER*8 LSCdate
      COMMON /LSCversn/ LSCdate
      CHARACTER*8 Cargumnt
      INTEGER Iargumnt
      REAL    Rargumnt
      INTEGER istring
      INTEGER i,j, idelta, jdelta
      CHARACTER*10 MyString
      CHARACTER*21 PlString
      CHARACTER*40 TiString
      DATA idelta, jdelta , istring / 340, 25, 0 /
      REAL    ZERO, ONE
      DATA    ZERO, ONE/
     ^         0.0, 1.0/
      write(PlString,'(a10,  i10,''$'')') MyString, Iargumnt
      go to 10
      ENTRY      GrafFltP ( MyString, Rargumnt )
      write(PlString,'(a10,f10.4,''$'')') MyString, Rargumnt
      go to 10
      ENTRY      GrafChrS ( MyString, Cargumnt )
      write(PlString,'(a10,2x,a8,''$'')') MyString, Cargumnt
10    continue
c
      if (MyString .eq. 'BEGIN PLOT' .or.
     ^    istring  .eq. 0                 ) then
	call EZinit
        call GNinit(nTSCgraf)
        call EZsets(1, 1023, 1, 768,ZERO, ONE,ZERO, ONE, 1) 
        call GNsets(1, 1023, 1, 768,ZERO, ONE,ZERO, ONE, 1) 
        write(TiString,
     ^  '(''LSC_'',a8, '' inputs and parameters:$'')') LSCdate
        call EZwrit(100, 735,TiString,0,0)
        call GNlist(100, 735,TiString,0,0)
	j = 700
	i =  20
        istring = 1
        return
      endif
c
      if (MyString .eq. 'END PLOT  ') then
        istring = 0
        call EZfini(0,0)
        call GNfinL
        return
      endif
c
      call EZwrit(i, j, PlString, 0,0)
      call GNlist(i, j, PlString, 0,0)
      istring = istring + 1
      j = j - jdelta
      if (j .le. jdelta ) then
        j = 700
        i = i + idelta
      endif
c
      if (i .ge. 3*idelta) then
        call EZfini(0,0)
        call GNfinL
        istring = 0
      endif
c
      return
      END


c
c                                                                      |
c                                                                      |
c     AcDc                                  ---------------------------| 









